"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = void 0;
var schematics_1 = require("@angular-devkit/schematics");
var workspace_1 = require("@schematics/angular/utility/workspace");
var core_1 = require("@angular-devkit/core");
function selectTargetFile(translation) {
    if (!translation || translation.length === 1) {
        return translation === null || translation === void 0 ? void 0 : translation[0];
    }
    // simple heuristic: shortest path (translations from third parties are probably from node_modules..)
    var sorted = __spreadArray([], translation, true).sort(function (a, b) { return a.length === b.length ? 0 : (a.length > b.length ? 1 : -1); });
    return sorted[0];
}
function getFormatFromTargetFile(targetFilePath, tree, context) {
    var _a;
    if (targetFilePath) {
        var content = (_a = tree.read(targetFilePath)) === null || _a === void 0 ? void 0 : _a.toString();
        var m = content === null || content === void 0 ? void 0 : content.match(/<xlff[^>]*"version"="([^"])"/i);
        if (m) {
            switch (m[1]) {
                case '1.2':
                    return 'xlf';
                case '2.0':
                    return 'xlf2';
                default:
                    context.logger.warn("unexpected xliff version in ".concat(targetFilePath, ": ").concat(m[1]));
                    return undefined;
            }
        }
    }
    return undefined;
}
// noinspection JSUnusedGlobalSymbols
function ngAdd(_options) {
    return function (tree, context) {
        return (0, workspace_1.updateWorkspace)(function (workspace) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
            var projectName = _options.project || Array.from(workspace.projects.keys())[0];
            var projectWorkspace = workspace.projects.get(projectName);
            if (!projectWorkspace) {
                throw new schematics_1.SchematicsException("Project ".concat(projectName, " not found!"));
            }
            // infer target files:
            var i18nExtension = projectWorkspace.extensions.i18n;
            // alternative: search tree for *.xlf? --> not performant, contains node_modules
            var files = (i18nExtension === null || i18nExtension === void 0 ? void 0 : i18nExtension.locales) ? (_a = Object.values(i18nExtension === null || i18nExtension === void 0 ? void 0 : i18nExtension.locales)) === null || _a === void 0 ? void 0 : _a.map(function (l) { var _a; return selectTargetFile((_a = l) === null || _a === void 0 ? void 0 : _a.translation); }).filter(function (f) { return f !== undefined; }) : undefined;
            if (!(files === null || files === void 0 ? void 0 : files.length)) {
                context.logger.warn('Could not infer translation target files, please setup angular i18n and re-run `ng add ng-extract-i18n-merge`: https://angular.io/guide/i18n-common-merge#define-locales-in-the-build-configuration');
            }
            else {
                context.logger.info('Found target translation files: ' + JSON.stringify(files));
            }
            // infer outputPath
            var outputPathFromExtractI18nOptions = (_c = (_b = projectWorkspace.targets.get('extract-i18n')) === null || _b === void 0 ? void 0 : _b.options) === null || _c === void 0 ? void 0 : _c.outputPath;
            var outputPathFromTargetFiles = (_d = files === null || files === void 0 ? void 0 : files[0]) === null || _d === void 0 ? void 0 : _d.substring(0, (_f = (_e = files === null || files === void 0 ? void 0 : files[0]) === null || _e === void 0 ? void 0 : _e.lastIndexOf('/')) !== null && _f !== void 0 ? _f : (_g = files === null || files === void 0 ? void 0 : files[0]) === null || _g === void 0 ? void 0 : _g.length);
            var outputPath = (_h = outputPathFromExtractI18nOptions !== null && outputPathFromExtractI18nOptions !== void 0 ? outputPathFromExtractI18nOptions : outputPathFromTargetFiles) !== null && _h !== void 0 ? _h : 'src/locales';
            context.logger.info("inferred output path: ".concat(outputPath));
            // check if inferred matches "extract-i18n" target config
            // infer format:
            var formatFromExtractI18nOptions = (_k = (_j = projectWorkspace.targets.get('extract-i18n')) === null || _j === void 0 ? void 0 : _j.options) === null || _k === void 0 ? void 0 : _k.format;
            var formatFromTargetFiles = getFormatFromTargetFile(files === null || files === void 0 ? void 0 : files[0], tree, context);
            var format = (_l = formatFromExtractI18nOptions !== null && formatFromExtractI18nOptions !== void 0 ? formatFromExtractI18nOptions : formatFromTargetFiles) !== null && _l !== void 0 ? _l : 'xlf2';
            context.logger.info("inferred format: ".concat(format));
            // remove path from files
            var filesWithoutOutputPath = files === null || files === void 0 ? void 0 : files.map(function (f) { return (0, core_1.relative)("/".concat(outputPath), "/".concat(f)); });
            var target = projectWorkspace.targets.get('extract-i18n-merge');
            var builderOptions = { format: format, outputPath: outputPath, targetFiles: filesWithoutOutputPath !== null && filesWithoutOutputPath !== void 0 ? filesWithoutOutputPath : [] };
            if (target) {
                context.logger.info("Overwriting previous extract-i18n-merge entry in project ".concat(projectName, "."));
                target.options = builderOptions;
            }
            else {
                projectWorkspace.targets.add({
                    name: 'extract-i18n-merge',
                    builder: 'ng-extract-i18n-merge:ng-extract-i18n-merge',
                    options: builderOptions,
                });
            }
            if (_options.packageScript) {
                var packageJson = JSON.parse(tree.read('package.json').toString('utf8'));
                packageJson.scripts = __assign(__assign({}, packageJson.scripts), { 'extract-i18n-merge': "ng run ".concat(projectName, ":extract-i18n-merge") });
                tree.overwrite('package.json', JSON.stringify(packageJson, null, 2));
            }
        });
    };
}
exports.ngAdd = ngAdd;
